"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function slashed(path) {
    return (path !== '/') ? (path.endsWith("/") ? path.slice(0, -1) : path) : '/';
}
exports.slashed = slashed;
function addVersionToRoute(version, route) {
    return `/v${versionNumberToObject(version).major}` + route;
}
exports.addVersionToRoute = addVersionToRoute;
function versionNumberToObject(version) {
    const regEx = /^v?(\d{1,2})(?:.(\d{1,2}|\*))?(?:.(\d{1,2}|\*))?/ig;
    let versionObject = { major: 1, minior: 0, release: 0 };
    let versionRegex = regEx.exec(version);
    if (versionRegex) {
        versionObject.major = versionRegex[1] || 1;
        versionObject.minior = versionRegex[2] || 0;
        versionObject.release = versionRegex[3] || 0;
    }
    return versionObject;
}
exports.versionNumberToObject = versionNumberToObject;
function isAcceptableVersion(routeVersion, allowedVersions) {
    if (!allowedVersions)
        return true;
    for (let _version of allowedVersions) {
        let _allowedVersion = versionNumberToObject(_version);
        let _routeVersion = versionNumberToObject(routeVersion);
        let releaseAcceptable = (_routeVersion.release === _allowedVersion.release || _allowedVersion.release === '*');
        let miniorAcceptable = (_routeVersion.minior === _allowedVersion.minior || _allowedVersion.minior === '*');
        let majorAcceptable = (_routeVersion.major === _allowedVersion.major);
        if (releaseAcceptable && miniorAcceptable && majorAcceptable)
            return true;
    }
    return false;
}
exports.isAcceptableVersion = isAcceptableVersion;
