"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const errors_1 = require("./errors");
class RequestHandler {
    constructor(ApiServer, request, response) {
        this.ApiServer = ApiServer;
        this.request = request;
        this.response = response;
        this._middlewareCount = 0;
        this._middleware = [];
        this.route();
        if (this.request.route) {
            this.middleware();
            this.handle();
        }
        else {
            response.send(new errors_1.ResourceNotFoundError);
        }
    }
    middleware() {
        let route = this.request.route;
        this._middleware = [];
        this._middleware = this.ApiServer.middleware.slice(0);
        if (route.middleware)
            this._middleware = this._middleware.concat(route.middleware);
    }
    handle(...args) {
        let route = this.request.route;
        if (this._middlewareCount < this._middleware.length && this._middleware[this._middlewareCount]) {
            this._middlewareCount++;
            return this._middleware[this._middlewareCount - 1](this.request, this.response, this.handle.bind(this));
        }
        if (route && typeof route.service === 'function') {
            return route.service(this.request, this.response);
        }
        else {
            this.response.send(new errors_1.InternalError);
            throw new Error('Route service must be a function');
        }
    }
    route() {
        return this.ApiServer.routes.find(route => {
            let version = helpers_1.versionNumberToObject(route.version || '1.0.0');
            let requestUrl = this.request.url;
            let path = helpers_1.slashed(route.path);
            let regex = new RegExp(path.replace(/:[^\s/]+/g, "([^/\]+)"));
            let matches = helpers_1.slashed(requestUrl.split("?")[0]).match(regex);
            let params = path.match(/:[^\s/]+/g);
            if (matches && matches[0] === matches["input"] && (route.method !== '*') || (route.method === this.request.method)) {
                for (let k in params) {
                    this.request.params[params[k].slice(1)] = decodeURI(matches[parseInt(k) + 1]);
                }
                this.request.route = route;
                return true;
            }
        });
    }
    static handle(ApiServer, request, response) {
        return new RequestHandler(ApiServer, request, response);
    }
}
exports.RequestHandler = RequestHandler;
