import {
    IncomingMessage,
    ServerResponse
} from 'http';

// Server interfaces
export interface CorsServerOption {
    origin?: string;
    maxAge?: number;
    credentials?: boolean;
}

export interface ServerOptions {
    port: number;
    name?: string;
    hostname?: string;
    serverName?: string;
    enableSecurityHeaders?: boolean;
    cors?: CorsServerOption;
    versions?: string[];
}

// Request intergaces
export interface Request extends IncomingMessage {
    query: any;
    path?: string;
    params?: any;
    body?: any;
    route?: Object;
    middlewareParams?: any[];
}

// Response
export interface Response extends ServerResponse {
    status?(code: number): void;
    send?(data: any, status?: number): void;
    return?(code: number, data: any): void;
}

// Route
export interface Route {
    method: string | string[];
    path: string;
    version?: string;
    middleware?: Function[] | Function;
    service(request: Request, response: Response): void;
}

// Middleware
interface Middleware {
    request: Request;
    response: Response;
    next: Function;
}


export declare class ApiServer {


    constructor(options: ServerOptions);

    register(routes: Route[]): void;

    enableLogger(): void;

    run(): void;

    use(middleware: Function): Middleware;    

}

export function bodyParser();

// Errors
export namespace Errors {
    class ApiError {
        private status: number;
        private message: string;
        private description; string;
    }

    // 4xx
    export class NotAuthorizedError extends ApiError {}
    export class ResourceNotFoundError extends ApiError {}
    export class InvalidCredentialsError extends ApiError {}
    export class MissingParameterError extends ApiError {}

    // 5xx
    export class InternalError extends ApiError {}
}


export declare let b: any;