export function slashed(path: string): string {
    return (path !== '/') ? (path.endsWith("/") ? path.slice(0, -1) : path): '/';
}

// Versioning
export function addVersionToRoute(version: string, route: string) {
    let versionObject: any = versionNumberToObject(version);

    return `/v${versionObject.major}` + route;
}

export function versionNumberToObject(version: string) {
    const regEx = /^v?(\d{1,2})(?:.(\d{1,2}|\*))?(?:.(\d{1,2}|\*))?/ig;

    let versionObject: any = { major: 1, minior: 0, release: 0 };
    let execRegex = regEx.exec(version);

    if (execRegex) {
        versionObject.major = execRegex[1] || 1;
        versionObject.minior = execRegex[2] || 0;
        versionObject.release = execRegex[3] || 0;
    }

    return versionObject;
}

export function isAcceptableVersion(routeVersion: string, allowedVersions: string[]) {
    if (!allowedVersions) return true;
    
    for (let _version of allowedVersions) {
        let _allowedVersion = versionNumberToObject(_version);
        let _routeVersion = versionNumberToObject(routeVersion);
        
        let releaseAcceptable = (_routeVersion.release === _allowedVersion.release || _allowedVersion.release === '*');
        let miniorAcceptable = (_routeVersion.minior === _allowedVersion.minior || _allowedVersion.minior === '*');
        let majorAcceptable = (_routeVersion.major === _allowedVersion.major);
        
        if (releaseAcceptable && miniorAcceptable && majorAcceptable) return true;
    }

    return false;
}