import {
    IncomingMessage,
    ServerResponse
} from 'http';

// Server interfaces
export interface CorsServerOption {
    origin?: string;
    maxAge?: number;
    credentials?: boolean;
}

export interface ServerOptions {
    port: number;
    name?: string;
    hostname?: string;
    serverName?: string;
    enableSecurityHeaders?: boolean;
    cors?: CorsServerOption;
    versions?: string[];
}

// Request intergaces
export interface Request extends IncomingMessage {
    query: any;
    path?: string;
    params?: any;
    body?: any;
    route?: Object;
    middlewareParams?: any[];
}

// Response
export interface Response extends ServerResponse {
    status?(code: number): void;
    send?(data: any, status?: number): void;
    return?(code: number, data: any): void;
}

// Route
export interface Route {
    method: string | string[];
    
    path: string;
    
    version?: string;
    
    middleware?: Function[] | Function;

    service(request: Request, response: Response): void;
}